# Include common functions and configuration

. .\Configuration.ps1
. .\CommonFunctions.ps1

# Initialize variables

# Search site full URL
$searchSite = $rootUrl + $searchSitePath

# Profile pages host URL
$profilePages = $rootUrl + $profilePagesPath

# Check that the target search application exists and get its 
# localized name for localized SharePoints
$targetSearchApp = VerifySearchApplication($targetSearchApp)

# Function definitions

Function RemoveManagedProperty([string]$managedPropertyName)
{
  $ErrorActionPreference = "SilentlyContinue"
  $mp = Get-SPEnterpriseSearchMetadataManagedProperty -SearchApplication $searchapp -Identity $managedPropertyName
  if ($mp) {
    $mp | Remove-SPEnterpriseSearchMetadataManagedProperty -confirm:$false
    write-host "Removed managed property named $($managedPropertyName)."
  }
  else {
    write-host "Managed property named $($managedPropertyName) was not found."
  }
}
Function RemoveMetadataMapping([string]$managedPropertyName, [string]$crawledPropertyName)
{
  $ErrorActionPreference = "SilentlyContinue"
  $mp = Get-SPEnterpriseSearchMetadataManagedProperty -SearchApplication $searchapp -Identity $managedPropertyName
  $cp = Get-SPEnterpriseSearchMetadataCrawledProperty -SearchApplication $searchapp -Category $category -Name $crawledPropertyName
  if ($cp -AND $mp) {
    $mm = Get-SPEnterpriseSearchMetadataMapping -SearchApplication $searchapp -CrawledProperty $cp -ManagedProperty $mp
    if ($mm) {
      $mm | Remove-SPEnterpriseSearchMetadataMapping -confirm:$false
      write-host "Removed metadata mapping between properties $($managedPropertyName) and $($crawledPropertyName)."
    }
    else {
      write-host "Metadata mapping between properties $($managedPropertyName) and $($crawledPropertyName) was not found."
    }
  }
}

Function RemoveManagedProperties
{
  $searchapp = Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp
  $category = Get-SPEnterpriseSearchMetadataCategory -SearchApplication $searchapp Identity "Business Data" -ErrorAction SilentlyContinue
  if (!$category)
  {
    Write-Host -foregroundcolor Yellow Category `"Business Data`" was not found. This may be due to SharePoint being a localized version. Please see the list below for the available categories `
    and type in the translated equivalent for `"Business Data`":
    $categories = Get-SPEnterpriseSearchMetadataCategory -SearchApplication $searchapp
    $categories | Foreach-Object {$_.Name}
    $category = Read-Host -prompt "Translated category: "
  }
  RemoveManagedProperty "Type"
  RemoveManagedProperty "Context"
  RemoveManagedProperty "DetailsUrl"
  RemoveManagedProperty "IconUrl"
  RemoveManagedProperty "Symbol"
  RemoveManagedProperty "ChartUrl"
  RemoveManagedProperty "EditUrl"
  RemoveManagedProperty "MimeType"
  RemoveManagedProperty "AttachedText"
}

Function RemoveMetadataMappings
{
  $searchapp = Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp
  $category = Get-SPEnterpriseSearchMetadataCategory -SearchApplication $searchapp Identity "Business Data" -ErrorAction SilentlyContinue
  if (!$category)
  {
    Write-Host -foregroundcolor Yellow Category `"Business Data`" was not found. This may be due to SharePoint being a localized version. Please see the list below for the available categories `
    and type in the translated equivalent for `"Business Data`":
    $categories = Get-SPEnterpriseSearchMetadataCategory -SearchApplication $searchapp
    $categories | Foreach-Object {$_.Name}
    $category = Read-Host -prompt "Translated category: "
  }
  RemoveMetadataMapping "Type" "Entity.Type"
  RemoveMetadataMapping "Title" "Entity.Name"
  RemoveMetadataMapping "Author" "Entity.Author"
  RemoveMetadataMapping "Description" "Entity.Description"
  RemoveMetadataMapping "Symbol" "Entity.Symbol"
  RemoveMetadataMapping "Context" "Entity.Context"
  RemoveMetadataMapping "Context" "EmbeddedFile.Context"
  RemoveMetadataMapping "Context" "EmbeddedFileDocx.Context"
  RemoveMetadataMapping "Context" "EmbeddedFileXlsx.Context"
  RemoveMetadataMapping "Context" "EmbeddedFilePptx.Context"
  RemoveMetadataMapping "Context" "EmbeddedFilePdf.Context"
  RemoveMetadataMapping "Filename" "EmbeddedFile.Name"
  RemoveMetadataMapping "Filename" "EmbeddedFileDocx.Name"
  RemoveMetadataMapping "Filename" "EmbeddedFileXlsx.Name"
  RemoveMetadataMapping "Filename" "EmbeddedFilePptx.Name"
  RemoveMetadataMapping "Filename" "EmbeddedFilePdf.Name"
  RemoveMetadataMapping "FileExtension" "EmbeddedFile.Extension"
  RemoveMetadataMapping "FileExtension" "EmbeddedFileDocx.Extension"
  RemoveMetadataMapping "FileExtension" "EmbeddedFileXlsx.Extension"
  RemoveMetadataMapping "FileExtension" "EmbeddedFilePptx.Extension"
  RemoveMetadataMapping "FileExtension" "EmbeddedFilePdf.Extension"
  RemoveMetadataMapping "DetailsUrl" "Entity.DetailsUrl"
  RemoveMetadataMapping "DetailsUrl" "EmbeddedFile.DetailsUrl"
  RemoveMetadataMapping "DetailsUrl" "EmbeddedFileDocx.DetailsUrl"
  RemoveMetadataMapping "DetailsUrl" "EmbeddedFileXlsx.DetailsUrl"
  RemoveMetadataMapping "DetailsUrl" "EmbeddedFilePptx.DetailsUrl"
  RemoveMetadataMapping "DetailsUrl" "EmbeddedFilePdf.DetailsUrl"
  RemoveMetadataMapping "ChartUrl" "Entity.ChartUrl"
  RemoveMetadataMapping "EditUrl" "Entity.EditUrl"
  RemoveMetadataMapping "IconUrl" "Entity.IconUrl"
  RemoveMetadataMapping "Filename" "Entity.FileName"
  RemoveMetadataMapping "MimeType" "Entity.MimeType"
  RemoveMetadataMapping "AttachedText" "Entity.AttachedText"
  RemoveManagedProperties
}

Function RemoveBDCModel
{
  $context = $rootUrl
  $bdc = Get-SPBusinessDataCatalogMetaDataObject -BdcObjectType Catalog -ServiceContext $context
  $qprModel = $bdc.GetLobSystems("*") | Where-Object {$_.DefaultDisplayName -like "*Qpr.Model*"}
  foreach ($entity in $qprModel.Entities) { 
    if (!$entity.Active) { 
      $entity.Delete() }
    else {
      Write-Host -foregroundcolor red Cannot delete QPR BDC Model, some of its entities are in use
    }
  }
}



########## QPR Search Central setup script starts ##########

# Check if SharePoint snapin is loaded

Write-Host -foregroundcolor green Loading snap-ins...

Load-Snapins
Start-Service spadminv4

Write-Host -foregroundcolor green Cleaning crawl rules...

$trimmer = Get-SPEnterpriseSearchSecurityTrimmer -SearchApplication (Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp) | where {$_.crawlrulepath -like "*qpr_model_*"}
Remove-SPEnterpriseSearchSecurityTrimmer -SearchApplication (Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp) -Identity $trimmer
$rule = Get-SPEnterpriseSearchCrawlRule -SearchApplication (Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp) | where {$_.path -like "*qpr_model_*"}
Remove-SPEnterpriseSearchCrawlRule -SearchApplication (Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp) -Identity $rule

Write-Host -foregroundcolor green Removing solutions...

Uninstall-SPSolution -Identity "Qpr.Bdcmodel.wsp" 
Uninstall-SPSolution -Identity "Qpr.MasterPage.wsp"

stop-service spadminv4
start-spadminjob
start-service spadminv4

Remove-SPSolution -Identity "Qpr.Bdcmodel.wsp"
Remove-SPSolution -Identity "Qpr.MasterPage.wsp"

Write-Host -foregroundcolor green Checking if QPR features exist...

Get-SPFeature -limit all | where {$_.displayname -like "QPR*"} | uninstall-spfeature -identity {$_}

Write-Host -foregroundcolor green Removing sites...

Remove-SPSite -Identity $profilePages
Remove-SPSite -Identity $searchSite

Write-Host -foregroundcolor green Removing content source...

# Stop crawls if any are running
$contentSource = Get-SPEnterpriseSearchCrawlContentSource -Identity "QPR BDC Content Source" -SearchApplication (Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp)
$contentSource.StopCrawl()
do { Start-Sleep -s 2 } while ($contentSource.CrawlStatus -ne "Idle")

Remove-SPEnterpriseSearchCrawlContentSource -Identity "QPR BDC Content Source" -SearchApplication (Get-SPEnterpriseSearchServiceApplication -Identity $targetSearchApp)

$context = Get-SPSite $rootUrl | Get-SPServiceContext
$profileProvisioner = New-Object Microsoft.SharePoint.Portal.WebControls.ProfileProvisioner($context)
$res = $profileProvisioner.SetHostSiteUri("")

Write-Host -foregroundcolor green Removing metadata mappings...

RemoveMetadataMappings

Write-Host -foregroundcolor green Removing BDC model...

RemoveBDCModel

Write-Host -foregroundcolor green QPR Search Central uninstallation finished.